package com.ibm.commerce.multipleapprovals.install;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/**
 * This class installs the sample driver onto an installed version of
 * WebSphere Commerce Business Edition.
 */
public final class InstallDriver
 {
  /**
   * DB2 database type.
   */
  public final static String DATABASE_TYPE_DB2 = "DB2";

  /**
   * Oracle database type.
   */
  public final static String DATABASE_TYPE_ORACLE = "Oracle";

  /**
   * Standard input stream reader.
   */
  public InputStreamReader _in = new InputStreamReader(System.in);

  /**
   * Reference to the jar file that contains this class.
   */
  public ZipFile _zipFile = null;

  /**
   * Array of arguments passed to this class.
   */
  public String[] _args = null;

  /**
   * Location of WebSphere Commerce Server root directory.
   */
  public String _wcs = null;

  /**
   * Location of WebSphere Application Server root directory.
   */
  public String _was = null;

  /**
   * Name of the WebSphere Commerce instances that should be configured by this install.
   */
  public Vector _instances = new Vector();

  /**
   * Line separator character sequence for this platform.
   */
  public static String _newLine = System.getProperty("line.separator");

  /**
   * Operating system.
   */
  public static String _os = System.getProperty("os.name");

  /**
   * File separator.
   */
  public static String _fileSeparator = System.getProperty("file.separator");

  /**
   * Byte buffer used to unzip jar entries.
   */
  public byte[] _buffer = new byte[10000];

  /**
   * Character buffer used to modify text files.
   */
  public char[] _charBuffer = new char[10000];

  final static class InstanceInfo
   {
    String _name;
    String _configXML;
    String _databaseName;
    String _databaseUserId;
    String _databasePassword;
    String _databaseType;
    String _ear;
    String _wcaLogger;
    String _errorLogDir;
    String _nodeName;
   }

  /**
   * Constructor for this class.
   *
   * @param args An array of command line arguments.
   */
  public InstallDriver(String[] args)
   {
    _args = args;
    String jarFileName = null;
    try
     {
      jarFileName = getClass().getProtectionDomain().getCodeSource().getLocation().getFile();
     }
    catch (Exception e)
     {
     }
    while (jarFileName == null || jarFileName.trim().length() == 0)
     {
      System.out.println("Could not determine the name of the jar.");
      System.out.println("Please enter the jar name.");
      jarFileName = readLine();
     }

    try
     {
      _zipFile = new ZipFile(jarFileName.trim());
     }
    catch (Exception e)
     {
      System.out.println("Error reading jar file \"" + jarFileName + "\"");
     }
   }
  /**
   * Executes a specified command.
   *
   * @param command The command to be executed.
   */
  public static void executeCommand(String command)
   {
    System.out.println(command);
    try
     {
      Runtime runtime = Runtime.getRuntime();
      Process p = runtime.exec(command);
      InputStream in = p.getInputStream();
      int c = in.read();
      while (c != -1)
       {
        System.out.print((char)c);
        c = in.read();
       }
      InputStream err = p.getErrorStream();
      c = err.read();
      while (c != -1)
       {
        System.err.print((char)c);
        c = err.read();
       }
      int result = p.waitFor();
     }
    catch (Exception e)
     {
      System.out.println("Exception while executing "+command);
      System.out.println(e);
     }
   }
  public String getAttribute(StringBuffer content, String attribute)
   {
    int index = content.toString().indexOf(attribute);
    if (index != -1)
     {
      index += attribute.length() + 2;
      for (int i = index; i < content.length(); i++)
       {
        if (content.charAt(i) == '"')
         {
          return content.toString().substring(index, i);
         }
       }
     }
    return null;
   }
  public StringBuffer getFileContent(String fileName)
   {
    StringBuffer content = new StringBuffer();
    try
     {
      FileReader fileReader = new FileReader(fileName);
      int len = fileReader.read(_charBuffer);
      while (len > 0)
       {
        content.append(_charBuffer, 0, len);
        len = fileReader.read(_charBuffer);
       }
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println(e);
      e.printStackTrace();
     }
    return content;
   }
  /**
   * Locates the list of instances.
   */
  public void initInstances()
   {
    String wcs_instances = _wcs + _fileSeparator + "instances" + _fileSeparator + "wcs_instances";
    Vector instanceNames = new Vector();
    try
     {
      FileReader fileReader = new FileReader(wcs_instances);
      BufferedReader bufferedReader = new BufferedReader(fileReader);
      String line = bufferedReader.readLine();
      while (line != null)
       {
        if (line.startsWith("[instance]"))
         {
          line = bufferedReader.readLine();
          while (line != null)
           {
            if (line.trim().length() > 0)
             {
              instanceNames.addElement(line.substring(0, line.indexOf(";")));
             }
            line = bufferedReader.readLine();
           }
          break;
         }
        line = bufferedReader.readLine();
       }
      bufferedReader.close();
      fileReader.close();
     }
    catch (Exception e)
     {
      System.out.println("Error accessing \"" + wcs_instances + "\"");
      System.exit(0);
     }
    if (instanceNames.size() == 0)
     {
      System.out.println("No instances were found. Use Configuration Manager to create an instance.");
      System.exit(0);
     }
    for (int i = 0; i < instanceNames.size(); i++)
     {
      InstanceInfo instanceInfo = new InstanceInfo();
      _instances.addElement(instanceInfo);
      instanceInfo._name = (String) instanceNames.elementAt(i);
      instanceInfo._configXML = _wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "xml" + _fileSeparator + instanceInfo._name + ".xml";
      File instanceConfigFile = new File(instanceInfo._configXML);
      if (!instanceConfigFile.exists())
       {
        System.out.println("Could not locate instance configuration file \"" + instanceInfo._configXML + "\".");
        System.exit(0);
       }
      instanceInfo._ear = _was + "/installedApps/WC_Enterprise_App_" + instanceInfo._name + ".ear";
      instanceInfo._wcaLogger = _wcs + "/instances/" + instanceInfo._name + "/xml/loader/WCALoggerConfig.xml";
      instanceInfo._errorLogDir = _wcs + "/instances/" + instanceInfo._name + "/logs";

      StringBuffer content = getFileContent(instanceInfo._configXML);
      instanceInfo._databaseUserId = getAttribute(content, "DBUserID");
      if (instanceInfo._databaseUserId == null)
       {
        System.out.println("Could not determine database userId for instance " + instanceInfo._name);
        System.exit(0);
       }
      instanceInfo._databaseType = getAttribute(content, "DBMSName");
      if (instanceInfo._databaseType == null)
       {
        System.out.println("Could not determine database type for instance " + instanceInfo._name);
        System.exit(0);
       }
      if (instanceInfo._databaseType.equals(DATABASE_TYPE_DB2))
       {
        instanceInfo._databaseName = getAttribute(content, "  name");
       }
      else
       {
        instanceInfo._databaseName = getAttribute(content, "ServiceName");
       }
      if (instanceInfo._databaseName == null)
       {
        System.out.println("Could not determine database name for instance " + instanceInfo._name);
        System.exit(0);
       }
      content = getFileContent(_wcs + _fileSeparator + "instances" + _fileSeparator + instanceInfo._name + _fileSeparator + "xml" + _fileSeparator + instanceInfo._name + "_was.xml");
      instanceInfo._nodeName = getAttribute(content, "<node action=\"update\" name");
      if (instanceInfo._nodeName == null)
       {
        System.out.println("Could not determine node name for instance " + instanceInfo._name);
        System.exit(0);
       }
      while (true)
       {
        if (instanceInfo._databasePassword == null || instanceInfo._databasePassword.length() == 0)
         {
          System.out.println("Specify the password that should be used with user \"" + instanceInfo._databaseUserId + "\".");
          instanceInfo._databasePassword = readLine().trim();
         }
        else
         {
          break;
         }
       }
     }
   }
  /**
   * Locates and validate the WebSphere Application Server root directory.
   */
  public void initWAS()
   {
    if (isWindows())
     {
      String setenvFile = _wcs + "\\bin\\" + "setenv.bat";
      try
       {
        FileReader fileReader = new FileReader(setenvFile);
        BufferedReader bufferedReader = new BufferedReader(fileReader);
        String line = bufferedReader.readLine();
        while (line != null)
         {
          if (line.startsWith("SET WAS_HOME="))
           {
            _was = line.substring("SET WAS_HOME=".length());
            break;
           }
          line = bufferedReader.readLine();
         }
        bufferedReader.close();
        fileReader.close();
       }
      catch (Exception e)
       {
        System.out.println("Error accessing \"" + setenvFile + "\"");
        System.exit(0);
       }
      if (_was == null)
       {
        System.out.println("Unable to locate the WebSphere Application root directory.");
        System.exit(0);
       }
     }
    else if (isAIX())
     {
      _was = "/usr/WebSphere/AppServer";
     }
    else if (isSun() || isLinux())
     {
      _wcs = "/opt/WebSphere/AppServer";
     }
    else
     {
      System.out.println("Operating System could not be determined.");
      System.exit(0);
     }
    File installedAppsRoot = new File(_was + _fileSeparator + "installedApps");
    if (!installedAppsRoot.exists() || !installedAppsRoot.isDirectory())
     {
      System.out.println("Could not find root directory of WebSphere Application Server");
      System.exit(0);
     }
   }
  /**
   * Locates and validates the root directory for WebSphere Commerce.
   */
  public void initWCS()
   {
    if (isWindows())
     {
      if (_wcs == null)
       {
        _wcs = "C:\\Program Files\\WebSphere\\CommerceServer";
       }
      while (true)
       {
        if (_wcs != null && _wcs.length() > 0)
         {
          File instancesRoot = new File(_wcs + "\\instances");
          if (instancesRoot.exists() && instancesRoot.isDirectory())
           {
            break;
           }
         }
        System.out.println("The root directory for WebSphere Commerce Server was not found at: "+_wcs);
        System.out.println("Please specify the root directory for WebSphere Commerce Server.");
        _wcs = readLine().trim();
       }
     }
    else if (isAIX())
     {
      _wcs = "/usr/WebSphere/CommerceServer";
     }
    else if (isSun() || isLinux())
     {
      _wcs = "/opt/WebSphere/CommerceServer";
     }
    else
     {
      System.out.println("Operating System could not be determined.");
      System.exit(0);
     }
    File instancesRoot = new File(_wcs + _fileSeparator + "instances");
    if (!instancesRoot.exists() || !instancesRoot.isDirectory())
     {
      System.out.println("Could not find root directory of WebSphere Commerce Server 5.4.");
      System.exit(0);
     }
   }
  /**
   * Install the driver.
   */
  public void install()
   {
    if (_zipFile == null)
     {
      System.exit(0);
     }
    initWCS();
    initWAS();
    initInstances();
    installEntries();
    updateDataBase();
    System.out.println("Installation complete");
   }
  /**
   * Install the documents into their correct locations.
   * Entries that start with "ear" are installed in the installed enterprise application archive directory.
   * Entries that start with "wcs" are installed in the WebSphere Commerce Server root directory.
   */
  public void installEntries()
   {
    System.out.println("Installing files");
    Enumeration entries = _zipFile.entries();
    while (entries.hasMoreElements())
     {
      ZipEntry entry = (ZipEntry) entries.nextElement();
      if (!entry.isDirectory())
       {
        String name = entry.getName();
        if (name.startsWith("ear"))
         {
          for (int i = 0; i < _instances.size(); i++)
           {
            InstanceInfo instanceInfo = (InstanceInfo) _instances.elementAt(i);
            installEntry(entry, instanceInfo._ear + name.substring("ear".length()));
           }
         }
        else if (name.startsWith("wcs"))
         {
          installEntry(entry, _wcs + name.substring("wcs".length()));
         }
       }
     }
   }
  /**
   * Install a zip entry to a specified location.
   *
   * @param entry A ZipEntry object
   * @param targetName The target name of the file.
   */
  public void installEntry(ZipEntry entry, String targetName)
   {
    try
     {
      InputStream inputStream = _zipFile.getInputStream(entry);
      File targetFile = new File(targetName);
      targetFile.getParentFile().mkdirs();
      FileOutputStream outputStream = new FileOutputStream(targetName);
      int len = inputStream.read(_buffer);
      while (len != -1)
       {
        if (!isWindows() && !isBinary(targetName))
         {
          len = stripCarriageReturns(_buffer, len);
         }
        outputStream.write(_buffer, 0, len);
        len = inputStream.read(_buffer);
       }
      inputStream.close();
      outputStream.close();
      if (!isWindows() && targetName.endsWith(".sh"))
       {
        executeCommand("chmod 744 " + targetName);
       }
     }
    catch (Exception e)
     {
      System.out.println("exception while extracting "+targetName);
      System.out.println(e);
     }
   }
  /**
   * Return true if the operating system is AIX.
   */
  boolean isAIX()
   {
    return _os.toUpperCase().indexOf("AIX") != -1;
   }
  /**
   * Determine if the specified file should be treated as a binary file.
   *
   * @param filename The name of the file to be tested.
   *
   * @return <code>true</code> if the file is a binary file.
   */
  public static boolean isBinary(String filename)
   {
    return filename.endsWith(".class") ||
           filename.endsWith(".jar") ||
           filename.endsWith(".ear") ||
           filename.endsWith(".zip") ||
           filename.endsWith(".war");
   }
  /**
   * Return true if the operating system is Linux.
   */
  boolean isLinux()
   {
    return _os.toUpperCase().indexOf("LINUX") != -1;
   }
  /**
   * Return true if the operating system is Sun
   */
  boolean isSun()
   {
    return _os.toUpperCase().indexOf("SUN") != -1;
   }
  /**
   * Return true if the operating system is Windows.
   */
  boolean isWindows()
   {
    return _os.toUpperCase().indexOf("WINDOWS") != -1;
   }
  /**
   * Main entry point for this class.
   *
   * @param args An array of arguments.
   */
  public static void main(String[] args)
   {
    InstallDriver installDriver = new InstallDriver(args);
    installDriver.install();
    System.exit(0);
   }
  /**
   * Reads a line of text from standard input.
   *
   * @return A line of text entered by the user.
   */
  public String readLine()
   {
    String line = "";
    try
     {
      char c = (char) _in.read();
      while (c == '\n' || c == '\r')
       {
        c = (char) _in.read();
       }
      while (c != '\n' && c != '\r')
       {
        line += c;
        c = (char) _in.read();
       }
     }
    catch (Exception e2)
     {
     }
    return line;
   }
  /**
   * Strip "\r" characters from the specified buffer.
   *
   * @param buffer An array of bytes.
   * @param len Current length.
   *
   * @return New length.
   */
  public static int stripCarriageReturns(byte buffer[], int len)
   {
    for (int i = 0; i < len; i++)
     {
      if (buffer[i] == '\r')
       {
        System.arraycopy(buffer, i + 1, buffer, i, len - i);
        len--;
       }
     }
    return len;
   }
  /**
   * Update the commerce database.  This is done by invoking a bat file called
   * profileSamplePopulate.db2.bat in the WebSphere Commerce bin directory.
   */
  public void updateDataBase()
   {
    System.out.println("Updating database");
    Runtime runtime = Runtime.getRuntime();
    for (int i = 0; i < _instances.size(); i++)
     {
      InstanceInfo instanceInfo = (InstanceInfo) _instances.elementAt(i);
      String command;
      if (!isWindows())
       {
        if (instanceInfo._databaseType.equals(DATABASE_TYPE_DB2))
         {
          command = _wcs + "/bin/multipleApprovalsPopulatedb.db2.sh ";
         }
        else
         {
          command = _wcs + "/bin/multipleApprovalsPopulatedb.oracle.sh ";
         }
       }
      else // Windows
       {
        if (instanceInfo._databaseType.equals(DATABASE_TYPE_DB2))
         {
          command = _wcs + "\\bin\\multipleApprovalsPopulatedb.db2.bat ";
         }
        else
         {
          command = _wcs + "\\bin\\multipleApprovalsPopulatedb.oracle.bat ";
         }
       }
      command += instanceInfo._databaseName + " " +
                 instanceInfo._databaseUserId + " " +
                 instanceInfo._databasePassword + " " +
                 instanceInfo._databaseUserId.toUpperCase() + " " +
                 instanceInfo._errorLogDir + " " +
                 instanceInfo._wcaLogger;
      executeCommand(command);
     }
   }
  /**
   * Modify a string in a specified file.
   *
   * @param fileName The name of the file that is to be updated.
   * @param fromString The original string.
   * @param toString The replacement text.
   */
  public void updateFile(String fileName, String fromString, String toString)
   {
    try
     {
      StringBuffer content = getFileContent(fileName);
      int toIndex = content.toString().indexOf(toString);
      int index = content.toString().indexOf(fromString);
      if (toIndex == -1 && index > 0)
       {
        content.delete(index, index + fromString.length());
        content.insert(index, toString);
        FileWriter fileWriter = new FileWriter(fileName + ".new");
        fileWriter.write(content.toString());
        fileWriter.close();
        File oldFile = new File(fileName);
        oldFile.delete();
        File newFile = new File(fileName + ".new");
        newFile.renameTo(oldFile);
       }
     }
    catch (Exception e)
     {
      System.out.println(e);
      e.printStackTrace();
     }
   }
}
